import os.path
import os
from skconfig import Configuration, ConfigurationLocatorInfo
import shutil


# ------------------------------------------------------------------------------
#           check_specs_exist_in_user_space
# ------------------------------------------------------------------------------
def check_specs_exist_in_user_space():

    file_locator = ConfigurationLocatorInfo(packagename='altius_l1pp_preflight', groupname='altius_l1pp/usask-arg', yamlname='optics.yaml', location='user')
    try:
        config = Configuration(file_locator)
    except Exception:
        basedir, fname = os.path.split(__file__)
        sourcefolder = os.path.join(basedir, 'defaults')
        config = Configuration()
        filename = config.locatorinfo_to_filename(file_locator)
        destfolder, fname = os.path.split(filename)

        if not os.path.exists(destfolder):
            print('No user configuration found for altius_l1pp_preflight. Copying default configuration from {:s} to {:s}'.format(sourcefolder, destfolder))
            shutil.copytree(sourcefolder, destfolder)


# ------------------------------------------------------------------------------
#           load_specs
# ------------------------------------------------------------------------------
def load_specs(yamlname: str) -> Configuration:

    config = None
    file_locations = ('user', 'python')
    for file_location in file_locations:
        location = ConfigurationLocatorInfo(packagename='altius_l1pp_preflight', groupname='altius_l1pp/usask-arg', yamlname=yamlname, location=file_location)
        try:
            config = Configuration(location)
        except Exception:
            pass
        else:
            break

    if (config is None) or (config.values is None) or (len(config.values)) == 0:
        folder, aname = os.path.split(__file__)
        filename = os.path.normpath(os.path.join(folder, 'defaults', yamlname))
        config = Configuration(filename)
    assert (config.values is not None) and (len(config.values) > 0)
    return config
