from typing import Tuple
import numpy as np
from ...configuration import ALTIUS_INSTRUMENT_SPECS
from altius_l1pp_preflight.radiance_simulator.altius_bright_limb_signals.deprecated.instrument_altiusbrightlimb import Instrument_AltiusBrightLimb
from altius_l1pp_preflight.radiance_simulator.altius_bright_limb_signals.deprecated.simple_attitude_solution import ALTIUS_SimpleSpacecraft


# ------------------------------------------------------------------------------
#           bright_limb_exponential
# ------------------------------------------------------------------------------
def bright_limb_exponential(inst: ALTIUS_INSTRUMENT_SPECS, maxsignal=10000.0, lowerboundarysignal=10000.0, lowerboundarypixelwidth=50) -> Tuple[np.ndarray, np.ndarray]:
    """
    Returns a floating point raw image (2000,1504) using an exponential decay across 15 km decadal scale height
    """

    simpleattitude = ALTIUS_SimpleSpacecraft(inst, lowestheight=0.0)
    simpleattitude.position_and_orient_satellite(30.0, 40.0, 40.0)
    fovlist, heights = simpleattitude.make_list_of_pixel_fov()                                                          # Get the heights of
    numheights = heights.size
    numhoriz = inst.Nx
    decadalscaleheight = 15.0 * 1000.0                                                                                    # Decadal scale height of atmosphere in meters
    H = -decadalscaleheight / 2.3025850929940456840179914546844                                         # number of pixels for signal to fall by a factor of e
    h = np.zeros(numheights)
    hoffset = (heights - heights[lowerboundarypixelwidth])
    h[0:lowerboundarypixelwidth] = lowerboundarysignal
    h[lowerboundarypixelwidth:-1] = maxsignal * np.exp(hoffset[lowerboundarypixelwidth:-1] / H)                         # get the 1-D height porofile
    image = np.tile(h, numhoriz).reshape(numhoriz, numheights).transpose()                                                 # get the 2-D image
    return image, heights                                                                                               # return the image


# ------------------------------------------------------------------------------
#           bright_limb_simpleRT
# ------------------------------------------------------------------------------
def bright_limb_simpleRT(specs: ALTIUS_INSTRUMENT_SPECS, wavelen: float, lowest_heightkm=0.0):

    model = Instrument_AltiusBrightLimb(specs)
    model.set_lowestheight(lowest_heightkm * 1000.0)
    results = model.calculate_signals(wavelengths=wavelen)
    h = results['h']
    photons = results['photons']
    nphot = photons.size
    numhoriz = specs.Nx
    numheights = specs.Ny
    column = np.zeros([numheights])
    column[0:nphot] = photons[:, 0]
    image = np.tile(column, numhoriz).reshape(numhoriz, numheights).transpose()
    return image, h
