import numpy as np
from ..psf.psf_tools import modified_harvey_stray_light_bsdf, apply_vis_psf_vignetting_curve
from altius_l1pp_preflight.radiance_simulator.altius_bright_limb_signals.altius_simple_skimpy_model import make_altius_signal_vs_angle


# ------------------------------------------------------------------------------
#           make_vis_extended_scene
# ------------------------------------------------------------------------------
def make_vis_extended_scene(wavelen: float, albedo: float, harvey_coeffs=None, maximum_value: float = 14000.0):

    anglevert, anglehoriz, signal = make_altius_signal_vs_angle(wavelen, albedo)
    m = np.nanmax(signal)
    signal *= maximum_value / m

    psfangles = np.zeros([anglevert.size, anglehoriz.size])
    truesignal = np.zeros_like(psfangles)
    v2 = np.square(anglevert)
    h2 = np.square(anglehoriz)
    for i in range(anglehoriz.size):
        psfangles[:, i] = v2 + h2[i]
        truesignal[:, i] = signal
    psfangles = np.sqrt(psfangles)
    psf = modified_harvey_stray_light_bsdf(psfangles, harvey_coeffs=harvey_coeffs)
    vsignal = apply_vis_psf_vignetting_curve(truesignal, anglevert)
    return vsignal, psf, anglevert, anglehoriz