from typing import Tuple
import numpy as np
from .get_example_raw_srf import get_raw_srf
#------------------------------------------------------------------------------
#           altius_uv_OOB_srf
#------------------------------------------------------------------------------

def altius_uv_OOB_srf( centralwavelen_nm : float)->Tuple[np.ndarray,np.ndarray]:
    """
    Returns the out-of-band spectral stray light for the UV channel. The stray light depends
    upon the central wavelength of the UV channel.

    Inputs
    ------
    centralwavelen_nm : float
        The central srf_wavelen of the UV channel for which the out of band SRF is required.

    Returns
    -------
    Tuple[np.ndarray, np.ndarray]
        Returns the wavelength and out-of-band spectral response function as two 1-D arrays. The first element of the tuple
        is the wavelength in nanometers of the spectral response function.  The second element is the spectral response function
        which expresses the fraction of photons at a given wavelength that contribute to the signal measured at the central wavelength: it is
        a number between 0 and 1.

    """
    srfraw  = get_raw_srf()
    srftemp = np.reshape(srfraw, (srfraw.size// 2, 2) )
    srf_wavelen = np.copy(srftemp[:, 0])
    srf = np.copy(srftemp[:, 1] )
    return srf_wavelen, srf


