import scipy.io as io
import scipy.signal
import numpy as np
import math
import matplotlib.pyplot as plt
import scipy.interpolate

# psf_filename = "C:/Users/dougd/OneDrive/Desktop/psf_uv_300nm_EOL.mat"
psf_filename = r'C:\Users\nickl\Documents\Work\software\ARG_Instruments\QinetiQ\ancillary_data\PSF\2020-10-20-PSF\PSF\psf_uv_300nm_EOL.mat'

psfs = io.loadmat(psf_filename)
psf = psfs.get('psf_uv')


slope = 34/513

plt.figure(1)
plt.imshow(np.log(psf), aspect='auto', origin='lower')
# plt.show()


psf_horiz = psf[psf.shape[0]//2, :]
psf_horiz /= psf_horiz.sum()
num_angles = psf_horiz.shape[0]
min_angle = -num_angles//2*slope + slope*0.5
max_angle = -min_angle
psf_horiz_angles = np.linspace(min_angle, max_angle, num_angles)
delta_angle = psf_horiz_angles[1]-psf_horiz_angles[0]

min_angle = -60
max_angle = -min_angle

vignetting_angles = np.arange(min_angle, max_angle+delta_angle*0.001, delta_angle)
num_angles = vignetting_angles.shape[0]

vignetting_full = np.zeros([num_angles])

for angle_ctr in range(num_angles):
    if -28 <= vignetting_angles[angle_ctr] <= -17:
        vignetting_full[angle_ctr] = (vignetting_angles[angle_ctr]+28)*1.0/11.0
    if -17 <= vignetting_angles[angle_ctr] <= 17:
        vignetting_full[angle_ctr] = 1
    if 17 <= vignetting_angles[angle_ctr] <= 28:
        vignetting_full[angle_ctr] = -(vignetting_angles[angle_ctr]-28)*1.0/11.0

vignetting_out = np.zeros([num_angles])

for angle_ctr in range(num_angles):
    if -28 <= vignetting_angles[angle_ctr] <= -17:
        vignetting_out[angle_ctr] = (vignetting_angles[angle_ctr]+28)*1.0/11.0
    if -17 <= vignetting_angles[angle_ctr] <= 17:
        vignetting_out[angle_ctr] = 0
    if 17 <= vignetting_angles[angle_ctr] <= 28:
        vignetting_out[angle_ctr] = -(vignetting_angles[angle_ctr]-28)*1.0/11.0



plt.figure(2)
plt.plot(vignetting_angles, vignetting_out, 'b-', label='Out of Field')
plt.plot(vignetting_angles, vignetting_full, 'r-.', label='Whole Field')
plt.legend()
plt.xlabel('Horizontal angle, milli-Radians')
plt.ylabel('Vignetting')
plt.title("Vignetting Curves ")
# plt.show()


psf_horiz_full = np.copy(psf_horiz)
response_full = scipy.signal.convolve(vignetting_full, psf_horiz_full, mode='full')

psf_horiz_ghosts = np.copy(psf_horiz)
psf_horiz_ghosts[(psf_horiz_angles < - 43) | ((psf_horiz_angles > -32) & (psf_horiz_angles < 32)) | (psf_horiz_angles > 45)] = 0
response_ghosts = scipy.signal.convolve(vignetting_full, psf_horiz_ghosts, mode='full')

psf_horiz_near = np.copy(psf_horiz)
psf_horiz_near[(psf_horiz_angles < -4.2) | (psf_horiz_angles > 4.2) | ((psf_horiz_angles > -0.1) & (psf_horiz_angles < 0.1))] = 0
response_near = scipy.signal.convolve(vignetting_full, psf_horiz_near, mode='full')

psf_horiz_centre = np.copy(psf_horiz)
psf_horiz_centre[(psf_horiz_angles < -0.1) | (psf_horiz_angles > 0.1)] = 0
response_centre = scipy.signal.convolve(vignetting_full, psf_horiz_centre, mode='full')

psf_horiz_scatter = np.copy(psf_horiz)
psf_horiz_scatter[(psf_horiz_near != 0) | (psf_horiz_centre != 0) | (psf_horiz_ghosts != 0)] = 0
response_scatter = scipy.signal.convolve(vignetting_full, psf_horiz_scatter, mode='full')


min_angle = -response_full.shape[0]//2*slope +slope*0.5
max_angle = -min_angle
response_angles = np.arange(min_angle, max_angle+delta_angle*0.001, delta_angle)


plt.figure(4)
plt.plot(psf_horiz_angles, np.log10(psf_horiz_full))
plt.title("Horizontal PSF - Full")
plt.xlabel("Angle millirad")
plt.ylabel("log10(stray_psf)")
# plt.show()


plt.figure(5)
plt.plot(psf_horiz_angles, np.log10(psf_horiz_centre))
plt.title("Horizontal PSF - Only Central Sharp Peak")
plt.xlabel("Angle millirad")
plt.ylabel("log10(stray_psf)")
# plt.show()

plt.figure(6)
plt.plot(psf_horiz_angles, np.log10(psf_horiz_near))
plt.title("Horizontal PSF - Only Near Field")
plt.xlabel("Angle millirad")
plt.ylabel("log10(stray_psf)")
# plt.show()

plt.figure(7)
plt.plot(psf_horiz_angles, np.log10(psf_horiz_ghosts))
plt.title("Horizontal PSF - Only Ghosts")
plt.xlabel("Angle millirad")
plt.ylabel("log10(stray_psf)")
# plt.show()

plt.figure(8)
plt.plot(psf_horiz_angles, np.log10(psf_horiz_scatter))
plt.title("Horizontal PSF - Only Scatter")
plt.xlabel("Angle millirad")
plt.ylabel("log10(stray_psf)")
# plt.show()


response_full_centre = scipy.signal.convolve(vignetting_full, psf_horiz_centre, mode='full')
response_out_full = scipy.signal.convolve(vignetting_out, psf_horiz_full, mode='full')
response_out_near = scipy.signal.convolve(vignetting_out, psf_horiz_near, mode='full')
response_out_ghosts = scipy.signal.convolve(vignetting_out, psf_horiz_ghosts, mode='full')
response_out_scatter = scipy.signal.convolve(vignetting_out, psf_horiz_scatter, mode='full')


plt.figure(11)
plt.plot(response_angles, (response_out_full/response_full_centre))
plt.plot(response_angles, (response_out_near/response_full_centre))
plt.plot(response_angles, (response_out_scatter/response_full_centre))
plt.plot(response_angles, (response_out_ghosts/response_full_centre))
plt.yscale('log')
plt.xlim([-16.9, 16.9])
plt.ylim([1e-7, 1e-1])
plt.title("Fractional Contributions to Signal from Full PSF")
plt.xlabel("Angle millirads")
plt.ylabel("Fractional Signal")
plt.legend(["Full", "Near", "Scatter", "Ghosts"])
plt.show()

print("Done")
#
#
# pixel_size = 10
# distance_per_millirad = (513*pixel_size)/34
# millirad_per_distance = 1.0/distance_per_millirad
#
#
#
# ####################################################
# #
# # from here on psf_horiz will integrate to 1, not sum to 1
# #
# ###################################################
#
# psf_horiz_x = psf_horiz_angles*distance_per_millirad
# psf_horiz_delta_x = psf_horiz_x[1]-psf_horiz_x[0]
# psf_horiz /= psf_horiz_delta_x
# f_psf = scipy.interpolate.interp1d(psf_horiz_x, psf_horiz, bounds_error=False, fill_value=0)
#
# distance_range = pixel_size
# angle_range = distance_range*millirad_per_distance
#
# psf_half_width = -psf_horiz_x[0]
#
# distance_min = -distance_range/2
# distance_max = -distance_min
# angle_min = (distance_min - psf_half_width) * millirad_per_distance
# angle_max = (distance_max + psf_half_width) * millirad_per_distance
#
# num_integration_points = 10001
# delta_distance = (distance_max - distance_min) / (num_integration_points - 1)
# distance = np.arange(distance_min, distance_max + delta_distance * 0.0001, delta_distance)
#
# num_samples = 50001
# delta_angle = (angle_max - angle_min) / (num_samples - 1)
# angles = np.arange(angle_min, angle_max + delta_angle * 0.0001, delta_angle)
# f_sampled = np.zeros([angles.shape[0]])
#
#
# for angle, angle_ctr in zip(angles, range(angles.shape[0])):
#     distance_centre = angle * distance_per_millirad
#
#     f_sampled[angle_ctr] = f_psf(distance-distance_centre).sum() * delta_distance
#
# plt.figure(14)
# plt.plot(angles, np.log(f_sampled))
# plt.plot(angles, np.log(f_sampled/(f_sampled.sum()*(angles[1]-angles[0])/angle_range)))
# # plt.show()
#